/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program, see the file COPYING; if not, write
    to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
    MA 02139, USA.
*/
#include "CheapMatrix.h"
#include <stdio.h> // for tmpnam

using namespace CheapMatrix;

main()
{
	// In-memory matrix
	Matrix A(4,4);

	// Initialisation list
	A = 1, 5.0, 9.0, 0.0,
	    4, 7.0, 8.0, 9.0,
	    2, 3.0, 6.0, 9.0,
	    8, 6,   2,   4;

	// iostream.
	cout << "A:" << endl << A << endl;

	// access to an element (row, line)
	A(4,1) = 1.1; // 8 => 1.1

	// submatrix (the doubles in the initialisation list above) and operator
	A(1,3,2,4) -= 2;
	cout << "submatrix operation" << endl << A << endl;
	cout << "second line: " << A(2,all) << endl;
	cout << "2, 3 columns, line boundary" << endl << A(boundary,2,3) << endl;
	Vector indices(3); indices = 1, 3, 4;
	cout << "1, 3, 4 columns, 2, 3 rows" << endl << A(2,3, indices) << endl;
	cout << "1, 3, 4 columns, all rows" << endl << A(all, indices) << endl;

	// more fancy stuff
	Matrix B = A(3,4,2,4);
	B.flip_rows(1,2);
	B.flip_col_range(1,3);
	cout << "B:" << endl << B << endl << "A (not affected):" << endl << A << endl;

	// transpose and copy back into A (without side effect...)
	A(1,3,1,2).copy(B.transpose());
	cout << "more transfos:" << endl << A << endl;

	// Just a reshape example
	cout << "A reshaped to dimension (2,8): " << endl << A.reshape(2,8) << endl;

	// This allocates a 100x200 matrix (it scales well, this is really a small
	// example) and allows only 1000 elements in memory at the same time
	// (actually, the closest system page size multiple greater than
	// 1000*sizeof(element)).
	// The 4th argument is a file name to dump the matrix. The file is not
	// destroyed if its size matches exactly, so you can use it as binary
	// storage for your data. Just to show it can be done, tmpnam is used, and
	// the 5th argument set to true for auto-deletion of the file after use.
	// Note: You can also use:
	//     Matrix::temporary_buffer(1000);
	//     Matrix M = Matrix::temporary(100,200);
	Matrix M(100,200,1000,tmpnam(0),true);

	// It's better to run through the matrix column by column, as this is
	// the underlying storage convention (adopted for easy linking with
	// LAPACK, written in FORTRAN) and minimizes swapping.
	// Note, this next line would be more easily achieved by filling with a
	// generator, but it is just to show you the correct looping order.
	// You can measure the influence of swapping by changing this order
	for (int j=1; j<=100; j++) for (int i=1; i<=100; i++) M(i,j) = i - j;

	// Vectors are column vectors for operations. And they derive from
	// the matrix class, so can be manipulated as matrices as well.
	// Same comment. See mapper.cpp
	Vector V(200);
	for (int i=1; i<=200; i++) V(i) = i;

	// Matrix multiplication. You can measure the influence of swapping by loading
	// M fully in memory, using the hold() and release() methods.
	// M.hold();
	Matrix result = M * V;
	// M.release();

	// Uncomment this to get the result, or dump it to a file with ofstream.
	// cout << result;

	// What else? Well, read the doc. Ah, perhaps built-in arithmetic:
	Matrix Q(2,2);
	Q = 0,                           3.14159265358979323846,
	    -3.14159265358979323846/3.0, 3.14159265358979323846/2.0;
	cout << cos(Q);

	// The most important thing to remember is that all matrices are in fact
	// wrappers around underlying data, but maintain their own geometry.
	// Enjoy!
	// 2000 04 17 Nicolas Brodu, nicolas.brodu@free.fr
}
