/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program, see the file COPYING; if not, write
    to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
    MA 02139, USA.
*/

// This example illustrates the use of the optimizer functions
// on a (relatively) simple example: Fitting a sine to a noisy one.

#include "CM_util.h"
#include "CM_optimize.h"

using namespace CheapMatrix;

const int max_time = 100;                // set max time
const double period = 20;                // period
const double amplitude = 5;              // amplitude
const double phase = 0.1;                // phase
const int seed = 42;                     // have reproductible results

// Model to fit a sine
// The model is of the form S(t) = alpha * sin (omega * t + phi);
class SineModel : public OptimizeTarget
{
protected:
	// Evaluate error and gradients on this series
	MatrixType evaluate(const Matrix& data, const Matrix& target, EvaluateTarget what = EVALUATE_BOTH);
public:
	SineModel();
};

main()
{
	// Initialize random number generator
	randinit(seed);

	// Create a sine series
	double omega = 2.0 * 3.1415 / period;
	Vector series = amplitude * sin(omega * range(1,max_time) + phase);

	// Add gaussian noise
	series += randn(max_time);

	// Create a sine model and an optimizer
	SineModel sine;
	SCG scg;            // Use scaled conjugate gradients
	scg.option(1) = 80; // max iteration, see doc for other options
	scg.log(&cout);     // display progress info

	// optimize the model on the range to produce the series
	scg.optimize(sine, range(1,max_time), series);

	// Save the results, if you want to plot them (no plotter 'yet')
	sine.save("model.parameters");
	save(series, "noisy_series.values"); // also save the data
}


// Implementation of the model
SineModel::SineModel()
{
	gradients = zeros(3); // 3 parameters: alpha, phi, omega
	values = randn(3);    // Set initial values to something random
	// Tip: you can set references to part of the values
	// example: A = values(5,10).reshape(2,3);
	// Those references will be kept valid
}

// Evaluate error (least squares) and gradients on this series
// Error function is 0.5 * sum( (alpha * sin(omega * t + phi) - x_t)^2 )
MatrixType SineModel::evaluate(const Matrix& data, const Matrix& target, EvaluateTarget what)
{
	// Calculate common terms for error and gradients
	// omega * t + phi
	Vector om_t_phi = values(3) * data + values(2);
	// sin(omega * t + phi)
	Vector sin_om_t_phi = sin(om_t_phi);
	// alpha * sin(omega * t + phi) - x_t
	Vector diff = values(1) * sin_om_t_phi - target;
	// Calculate gradients if necessary
	if (what & OptimizeTarget::EVALUATE_GRADIENT) {
		// Derivative with respect to alpha
		gradients(1) = dot(diff, sin_om_t_phi);
		Vector alpha_cos_om_t_phi = values(1) * cos(om_t_phi);
		// Derivative with respect to phi
		gradients(2) = dot(diff, alpha_cos_om_t_phi);
		// Derivative with respect to omega
		gradients(3) = dot(diff, multiply(data,alpha_cos_om_t_phi));
	}
	// Calculate error if necessary
	if (what & OptimizeTarget::EVALUATE_ERROR) {
		return 0.5 * dot(diff, diff);  // error
	} else return 0;  // Value not used in principle
}

// Tip: you can call sine.gradcheck(range(1,max_time), series); to check
// that this implementation is correct.
