/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#ifndef CHEAP_MATRIX_UTIL_H
#define CHEAP_MATRIX_UTIL_H

#include "CheapMatrix.h"
#include <string>

// Utility functions for the CheapMatrix Package

namespace CheapMatrix {

/***********************************************************
GENERATOR UTILITIES
***********************************************************/

// Convenience functions that use a generator
Matrix zeros(int rows, int cols=1, size_t bsize=0, const string& filename="", bool auto_del=false);
Matrix ones(int rows, int cols=1, size_t bsize=0, const string& filename="", bool auto_del=false);
Matrix eye(int rows, int cols=1, size_t bsize=0, const string& filename="", bool auto_del=false);

// A replacement for matlab a:b:c utility
Vector range(MatrixType bound1, MatrixType bound2); // increment assumed = 1
Vector range(MatrixType bound1, MatrixType increment, MatrixType bound2);

/***********************************************************
COMMON MATH FUNCTIONS
***********************************************************/

// Sum all elements
MatrixType sum(const Matrix& mat);
// Sum along the dimension given
Matrix sum(const Matrix& mat, int dim);

// Hehe, we can define our min/max for elements, namespace => no collision!
inline MatrixType min(const MatrixType& a, const MatrixType& b)
{
	return (a<b)?a:b;
}
inline MatrixType max(const MatrixType& a, const MatrixType& b)
{
	return (a>b)?a:b;
}

// Now min/max Matrix utilities
// min/max of all elements
MatrixType min(const Matrix& mat);
MatrixType max(const Matrix& mat);
// Same for the dimension given
Matrix min(const Matrix& mat, int dim);
Matrix max(const Matrix& mat, int dim);


/***********************************************************
BASIC STATISTICS
***********************************************************/

// mean is sum / number of elements
MatrixType mean(const Matrix& mat);
// mean along the dimension given
Matrix mean(const Matrix& mat, int dim);

// var is variance
MatrixType var(const Matrix& mat);
// variance along the dimension given
Matrix var(const Matrix& mat, int dim);

// stdev is standard deviation
MatrixType stdev(const Matrix& mat);
// standard deviation along the dimension given
Matrix stdev(const Matrix& mat, int dim);


/***********************************************************
RANDOM NUMBERS
***********************************************************/

// All the random generator functions use the system random(), so
// might not be enough if that function is poorly implemented.
// Initialize the seed with a value taken from the internal clock, or 'seed'
// It will be called automatically by the other functions if needed
void randinit();
// Can specify a seed (hence enable to reproduce results)
void randinit(long seed);

// returns a MatrixType from a uniform distribution on [0,1)
MatrixType randu();
// returns a MatrixType from a gaussian distribution with mean 0, variance 1
MatrixType randn();

// Generator facilities
Matrix randu(int rows, int cols=1, size_t bsize=0, const string& filename="", bool auto_del=false);
Matrix randn(int rows, int cols=1, size_t bsize=0, const string& filename="", bool auto_del=false);


/***********************************************************
Input / Output
***********************************************************/

// Load and save in transpose form in a text file
// This is much faster than the operators for big matrices, since the
// looping is done column-major.
// Also, load will fill the matrix given if the dimension match, or use
// a temporary otherwise => you don't need to know the dimension before
// loading
bool load(Matrix &a, const string& filename);
bool save(const Matrix &a, const string& filename, int precision=10);

}
#endif
