/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include <unistd.h>
#include <stdio.h> // tmpnam
#include <string.h> // strcpy!
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include "CheapMatrix.h"
#include "CM_util.h" // we can use those internally!

// Don't pollute the standard namespace with our definitions
namespace CheapMatrix {

static size_t _temp_size = 0;
// Specify buffer size for temporary matrices. 0 <=> in memory
void Matrix::temporary_buffer(size_t buf_size = 0)
{
	_temp_size = buf_size;
}
// returns a temporary matrix. The file name, if used, was found with
// tmpnam and the file will be destroyed after use.
Matrix Matrix::temporary(int rows, int cols)
{
	return Matrix(rows, cols, _temp_size, _temp_size?tmpnam(0):"", true);
}
Vector Vector::temporary(int rows)
{
	return Vector(rows, _temp_size, _temp_size?tmpnam(0):"", true);
}

Matrix::ReferencedData::ReferencedData(const size_t tsize, const size_t bsize, const string& filename, bool auto_del)
: start(0), count(1), auto_delete(auto_del) {
	page_size = getpagesize();
	total_size = ((tsize<=0)?1:tsize)*sizeof(MatrixType);
	// round up to page limit
	total_size = ((total_size + page_size-1) / page_size) * page_size;
	if (filename.empty() || filename[0]==0) { // all in memory
		data = new MatrixType[total_size/sizeof(MatrixType)];
		assert(data);
		fd = -1;
		buf_size = total_size;
		buf_save = buf_size;
		file="";
		return;
	}
	buf_size = ((bsize<=0)?1:bsize)*sizeof(MatrixType);
	buf_size = ((buf_size + page_size-1) / page_size) * page_size;
	if (buf_size>total_size) buf_size = total_size;
	buf_save = buf_size;
	fd = open(filename.c_str(), O_RDWR|O_CREAT, 0644);
	assert(fd!=-1);
	off_t size = lseek(fd,0,SEEK_END);
	if (size!=total_size) {
		close(fd);
		fd = open(filename.c_str(), O_RDWR|O_CREAT|O_TRUNC, 0644);
		lseek(fd,total_size-1,SEEK_SET);
		char foo=0;
		int res = write(fd,&foo,1);
		assert(res!=-1);
		close(fd);
		fd = open(filename.c_str(), O_RDWR, 0644);
		assert(fd!=-1);
	}
	lseek(fd,0,SEEK_SET);
	data = (MatrixType*)mmap(0, buf_size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
	assert(data!=(MatrixType*)-1);
	file = filename;
}

void Matrix::ReferencedData::dec() {
	if (--count==0) {
		if (fd==-1) {// in memory
			if (data) delete data; data = 0;
			delete this;
			return;
		}
		if (data) {
			int res = msync((char*)data, buf_size, MS_SYNC|MS_INVALIDATE);
			assert(res!=-1);
			res = munmap((char*)data, buf_size);
			assert(res!=-1);
		}
		if (fd) close(fd);
		if (!file.empty()) {
			if (auto_delete) unlink(file.c_str());
			file = "";
		}
		data = 0; fd = 0;
		delete this;
	}
}
MatrixType& Matrix::ReferencedData::operator()(off_t pos) {
	pos *= sizeof(MatrixType);
	if (pos<0 || pos>=total_size) {assert(false); return *(MatrixType*)0;}
	// currently not in memory => move the sliding mmap'd window
	if ((pos<start) || (pos>=start+buf_size)) {
		int res = msync((char*)data, buf_size, MS_SYNC|MS_INVALIDATE);
		assert(res!=-1);
		res = munmap((char*)data, buf_size);
		assert(res!=-1);
		if (pos<buf_size/2) start = 0;
		else if (pos>=total_size-buf_size/2) start = total_size-buf_size;
		else {
			start = (pos - buf_size/2 + page_size - 1) / page_size * page_size;
			if (start>pos) start -= page_size;
		}
//		cout<<"pos="<<pos<<", start="<<start<<", buf_size="<<buf_size<<endl;
		data = (MatrixType*)mmap(0, buf_size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, start);
		assert(data!=(MatrixType*)-1);
	}
	return *(MatrixType*)((char*)data+pos-start);
}

// force mapping of region [begin, end], boundaries included
bool Matrix::ReferencedData::map_all(off_t begin, off_t end)
{
	if (buf_size == total_size) return true; // includes memory mode
	begin *= sizeof(MatrixType); end *= sizeof(MatrixType);
	int res = msync((char*)data, buf_size, MS_SYNC|MS_INVALIDATE);
	assert(res!=-1);
	res = munmap((char*)data, buf_size);
	assert(res!=-1);
	off_t restore_start = start;
	start = begin / page_size * page_size; // round begin down, and end up
	buf_size = (end + sizeof(MatrixType) + page_size - 1) / page_size * page_size - start;
	data = (MatrixType*)mmap(0, buf_size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, start);
	if (data!=(MatrixType*)-1) return true;
	buf_size = buf_save; start = restore_start;
	data = (MatrixType*)mmap(0, buf_size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, start);
	assert(data!=(MatrixType*)-1);
	return false;
}

// revert to buffer mode if applicable
bool Matrix::ReferencedData::map_buffer() {
	if (buf_size == buf_save) return true; // includes memory mode
	int res = msync((char*)data, buf_size, MS_SYNC|MS_INVALIDATE);
	assert(res!=-1);
	res = munmap((char*)data, buf_size);
	assert(res!=-1);
	buf_size = buf_save;
	data = (MatrixType*)mmap(0, buf_size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
	assert(data!=(MatrixType*)-1);
	return true;
}

// Not lazy enough to #include <list> for a simple chained list...
// ...and that way I have a better control!
class Matrix::TransfoList {
public:
	TransfoType type;
	TransfoList* next;
	int param1, param2;
	Vector vec;
	TransfoList(TransfoType t, TransfoList* n, int p1=0, int p2=0)
		: type(t), next(n), param1(p1), param2(p2) {}
	TransfoList(TransfoType t, TransfoList* n, const Vector& v)
		: type(t), next(n), param1(0), param2(0), vec(v.copy()) {}
	~TransfoList();
	// Self duplicate
	TransfoList* dup();
};

Matrix::TransfoList::~TransfoList()
{
	// Short list, assume stack is big enough for recursion...
	if (next) delete next;
	next=0;
}

// Self duplicate
Matrix::TransfoList* Matrix::TransfoList::dup()
{
	Matrix::TransfoList* ret = new TransfoList(type, 0, param1, param2);
	ret->vec = vec.copy();
	for (TransfoList* self=next, *it=ret; self; self=self->next, it=it->next) {
		it->next = new TransfoList(self->type, 0, self->param1, self->param2);
		it->next->vec = self->vec.copy();
	}
	return ret;
}

// get offset from data corresponding to those coord in our system
off_t Matrix::translate_coord(int r, int c) const {
	assert(r>0 && c>0 && r<=nrows && c<=ncols);
	// apply all the transformations
	for (TransfoList* transfo=coord_transfo; transfo; transfo=transfo->next)
		switch (transfo->type) {
			case Translate:
				r += transfo->param1; c += transfo->param2;
				break;
			case Transpose:
				{int tmp = r; r = c; c = tmp;}
				break;
			case Reshape:
				{int tmp1 = (c-1) * transfo->param2 + (r-1);
				int tmp2 = tmp1 / transfo->param1;
				r = tmp1 - tmp2 * transfo->param1 + 1;
				c = tmp2 + 1;}
				break;
			case FlipColRange:
				if (c>=transfo->param1 && c<=transfo->param2)
					c = transfo->param1 + transfo->param2 - c;
				break;
			case FlipRowRange:
				if (r>=transfo->param1 && r<=transfo->param2)
					r = transfo->param1 + transfo->param2 - r;
				break;
			case FlipCols:
				if (c == transfo->param1) c = transfo->param2;
				else if (c == transfo->param2) c = transfo->param1;
				break;
			case FlipRows:
				if (r == transfo->param1) r = transfo->param2;
				else if (r == transfo->param2) r = transfo->param1;
				break;
			case RowIndex:
				r = (int)(transfo->vec(r)); // check done at creation
				break;                // and also bound check anyway
			case ColIndex:
				c = (int)(transfo->vec(c)); // check done at creation
				break;                // and also bound check anyway
		}
	// now return data offset
	return (c-1) * leading_dim + (r-1);
}


// sub-matrix constructor
Matrix::Matrix(int r1, int r2, int c1, int c2, const Matrix* mat) {
	assert(mat); // protected function, we should know what we're doing
	assert(r1>0 && c1>0 && r2<=mat->rows() && c2<=mat->cols()); // range checking
	ref = mat->ref;
	if (ref) ref->inc();  // 'if' useless with the range assert, in theory
	coord_transfo = (mat->coord_transfo) ? mat->coord_transfo->dup() : 0;
	// add this transfo on top of the others if necessary
	if (r1!=1 || c1!=1)
		coord_transfo = new TransfoList(Translate, coord_transfo, r1-1, c1-1);
	nrows = r2 - r1 + 1;
	ncols = c2 - c1 + 1;
	leading_dim = mat->leading_dim;
}
// sub-vector constructor
Vector::Vector(int r1, int r2, const Vector* vec)
{
	assert(vec); // protected function, we should know what we're doing
	assert(r1>0 && r2<=vec->rows()); // range checking
	ref = vec->ref;
	if (ref) ref->inc();
	coord_transfo = (vec->coord_transfo) ? vec->coord_transfo->dup() : 0;
	// add this transfo on top of the others if necessary
	if (r1!=1)
		coord_transfo = new TransfoList(Translate, coord_transfo, r1-1, 0);
	nrows = r2 - r1 + 1;
	ncols = 1;
	leading_dim = vec->leading_dim;
}

// destructor job
void Matrix::destroy() {
	if (ref) {
		ref->dec(); ref=0;
	}
	if (coord_transfo) {
		delete coord_transfo;
		coord_transfo=0;
	}
}

void Matrix::shallow_copy(const Matrix& target) {
	if (this == &target) return;
	if (ref != target.ref) {
		if (ref) ref->dec();
		ref = target.ref;
		if (ref) ref->inc();
	}
	coord_transfo = target.coord_transfo ? target.coord_transfo->dup() : 0;
	nrows = target.nrows;
	ncols = target.ncols;
	leading_dim = target.leading_dim;
}

// Create a rows*cols Matrix, either in memory or using a buffer of bsize
// elements and dumping the rest to the file filename is specified.
// The file will be created if it doesn't exist, and deleted automatically
// if auto_del is set to true.
Matrix::Matrix(int rows, int cols, size_t bsize, const string& filename, bool auto_del)
: coord_transfo(0), nrows(rows), ncols(cols), leading_dim(rows) {
	assert(nrows>0 && ncols>0);
	ref = new ReferencedData(nrows*ncols, bsize, filename, auto_del);
	// remove the assert, it will just be a null Matrix
	// assert(ref);
}

// data copy
Matrix Matrix::copy(const Matrix& target) const {
	assert(nrows==target.nrows && ncols==target.ncols);
	if (!target) return Matrix(); // Null matrix copying another!
	if (ref != target.ref) {
		// move colum-wise in the hope at least one of (*this, target) has
		// not been transposed => minimize buffer reloads
		for (int j=1; j<=ncols; j++) for (int i=1; i<=nrows; i++)
			(*this)(i,j) = target(i,j);
		return *this;
	}
	// possible overlap, use a temporary buffer
	Matrix tmp(nrows, ncols, _temp_size, _temp_size?tmpnam(0):"", true);
	// move colum-wise
	for (int j=1; j<=ncols; j++) for (int i=1; i<=nrows; i++)
		tmp(i,j) = target(i,j);
	for (int j=1; j<=ncols; j++) for (int i=1; i<=nrows; i++)
		(*this)(i,j) = tmp(i,j);
	return *this;
}

// copy ourselve
Matrix Matrix::copy() const
{
	if (!(*this)) return Matrix();
	// Use a whole new matrix
	Matrix tmp(nrows, ncols, _temp_size, _temp_size?tmpnam(0):"", true);
	return tmp.copy(*this);
}

// Detach ourselve to have our own copy of the data we refer to.
// Can dump into a file if specified
bool Matrix::detach(size_t bsize, const string& filename, bool auto_del)
{
	if (!(*this)) return true; // easy to detach a null matrix!
	// Use a whole new matrix
	Matrix tmp(nrows, ncols, bsize, filename, auto_del);
	if (!tmp) return false;
	for (int j=1; j<=ncols; j++) for (int i=1; i<=nrows; i++)
		tmp(i,j) = (*this)(i,j); // At least, column-wise is best for tmp
	// drop our data and refer to the copy (shallow copy)
	(*this) = tmp;
	return true;
}

// Fill the matrix vith a value
Matrix Matrix::fill(const MatrixType& value) const
{
	for (int j=1; j<=ncols; j++) for (int i=1; i<=nrows; i++)
		(*ref)(translate_coord(i,j)) = value;
	return *this; // include nullity case
}
// Fill the matrix using a generator
Matrix Matrix::fill(MatrixType (*func)(int,int)) const
{
	for (int j=1; j<=ncols; j++) for (int i=1; i<=nrows; i++)
		(*ref)(translate_coord(i,j)) = func(i,j);
	return *this; // include nullity case
}

// Map a function to all our elements
Matrix Matrix::map(MatrixType (*func)(MatrixType)) const
{
	for (int j=1; j<=ncols; j++) for (int i=1; i<=nrows; i++) {
		MatrixType& value = (*ref)(translate_coord(i,j));
		value = func(value);
	}
	return *this; // include nullity case
}
Matrix Matrix::map(MatrixType (*func)(MatrixType,void*), void* arg) const
{
	for (int j=1; j<=ncols; j++) for (int i=1; i<=nrows; i++) {
		MatrixType& value = (*ref)(translate_coord(i,j));
		value = func(value, arg);
	}
	return *this; // include nullity case
}

Matrix Matrix::reshape(const int r, const int c) const
{
	assert(r*c == nrows*ncols);
	if (r==nrows) return *this; // already in the correct shape.
	if (!(*this)) return Matrix(); // if the user specified c = 0...
	// create a clone first, without changing the transfo list
	Matrix res(1, nrows, 1, ncols, this);
	// add this transfo on top of the others if necessary
	if (res.coord_transfo && res.coord_transfo->type==Reshape)
		res.coord_transfo->param2 = r; // reuse the last reshape.
	else res.coord_transfo = new TransfoList(Reshape, res.coord_transfo, nrows, r);
	res.nrows=r; res.ncols=c;
	return res;
}

Matrix Matrix::transpose() const
{
	if (!(*this)) return Matrix();
	// create a clone first, without changing the transfo list
	Matrix res(1, nrows, 1, ncols, this);
	// add this transfo on top of the others if necessary
	if (res.coord_transfo && res.coord_transfo->type==Transpose) {
		// cancel it
		TransfoList * sav = res.coord_transfo;
		res.coord_transfo = res.coord_transfo->next;
		sav->next=0; // prevent recursive delete
		delete sav;
	}
	else res.coord_transfo = new TransfoList(Transpose, res.coord_transfo);
	int tmp=res.nrows; res.nrows=res.ncols; res.ncols=tmp;
	return res;
}

Matrix Matrix::flip_row_range(int r1, int r2) const
{
	assert(r1>=1 && r1<=r2 && r2<=nrows);
	// create a clone first, without changing the transfo list
	Matrix res(1, nrows, 1, ncols, this);
	// Cancel the previous transfo if it is the same!
	if (res.coord_transfo && res.coord_transfo->type==FlipRowRange
	&& ((res.coord_transfo->param1==r1 && res.coord_transfo->param2==r2)
	    || (res.coord_transfo->param1==r2 && res.coord_transfo->param2==r1))) {
		TransfoList * sav = res.coord_transfo;
		res.coord_transfo = res.coord_transfo->next;
		sav->next=0; // prevent recursive delete
		delete sav;
	}
	else res.coord_transfo = new TransfoList(FlipRowRange, res.coord_transfo, r1, r2);
	return res;
}
Matrix Matrix::flip_col_range(int c1, int c2) const
{
	assert(c1>=1 && c1<=c2 && c2<=ncols);
	// create a clone first, without changing the transfo list
	Matrix res(1, nrows, 1, ncols, this);
	// Cancel the previous transfo if it is the same!
	if (res.coord_transfo && res.coord_transfo->type==FlipColRange
	&& ((res.coord_transfo->param1==c1 && res.coord_transfo->param2==c2)
	    || (res.coord_transfo->param1==c2 && res.coord_transfo->param2==c1))) {
		TransfoList * sav = res.coord_transfo;
		res.coord_transfo = res.coord_transfo->next;
		sav->next=0; // prevent recursive delete
		delete sav;
	}
	else res.coord_transfo = new TransfoList(FlipColRange, res.coord_transfo, c1, c2);
	return res;
}
Matrix Matrix::flip_rows(int r1, int r2) const
{
	assert(r1>=1 && r1<=r2 && r2<=nrows);
	// create a clone first, without changing the transfo list
	Matrix res(1, nrows, 1, ncols, this);
	// Cancel the previous transfo if it is the same!
	if (res.coord_transfo && res.coord_transfo->type==FlipRows
	&& ((res.coord_transfo->param1==r1 && res.coord_transfo->param2==r2)
	    || (res.coord_transfo->param1==r2 && res.coord_transfo->param2==r1))) {
		TransfoList * sav = res.coord_transfo;
		res.coord_transfo = res.coord_transfo->next;
		sav->next=0; // prevent recursive delete
		delete sav;
	}
	else res.coord_transfo = new TransfoList(FlipRows, res.coord_transfo, r1, r2);
	return res;
}
Matrix Matrix::flip_cols(int c1, int c2) const
{
	assert(c1>=1 && c1<=c2 && c2<=ncols);
	// create a clone first, without changing the transfo list
	Matrix res(1, nrows, 1, ncols, this);
	// Cancel the previous transfo if it is the same!
	if (res.coord_transfo && res.coord_transfo->type==FlipCols
	&& ((res.coord_transfo->param1==c1 && res.coord_transfo->param2==c2)
	    || (res.coord_transfo->param1==c2 && res.coord_transfo->param2==c1))) {
		TransfoList * sav = res.coord_transfo;
		res.coord_transfo = res.coord_transfo->next;
		sav->next=0; // prevent recursive delete
		delete sav;
	}
	else res.coord_transfo = new TransfoList(FlipCols, res.coord_transfo, c1, c2);
	return res;
}

off_t Matrix::contiguous() const {
	if (!(*this)) return 0; // contiguous with null dim, or not contiguous...
	bool submatrix = true; // is always contiguous with no transfo, but in some cases we can know
	for (TransfoList* it=coord_transfo; it; it=it->next) {
		if (it->type != Translate) {submatrix = false; break;}
	}
	if (submatrix) return leading_dim;
	// Another case we can be sure: reshaping of a fully contiguous zone
	if (coord_transfo->type==Reshape && coord_transfo->param1==leading_dim) {
		bool other_are_subs = true;
		for (TransfoList* it=coord_transfo->next; it; it=it->next) {
			if (it->type != Translate) {other_are_subs = false; break;}
		}
		if (other_are_subs) return nrows;
	}
	// The only way to know for sure is to try...
	off_t cur_off = translate_coord(1,1);
	for (int i=2; i<=nrows; i++)
		if (++cur_off != translate_coord(i,1)) return 0;
	if (ncols==1) return nrows;
	off_t next_off = translate_coord(1,2);
	if (cur_off > next_off) return 0; // order reversed!
	off_t depl = next_off - cur_off;
	for (int j=2; j<=ncols; j++) {
		cur_off = translate_coord(1,j);
		for (int i=2; i<=nrows; i++)
			if (++cur_off != translate_coord(i,j)) return 0;
		if (j<ncols)
			if (cur_off+depl != translate_coord(1,j+1)) return 0;
	}
	// all columns are contiguous and there is the same depl between them
	return nrows + depl - 1;
}

bool Matrix::hold() const {
	if (!(*this)) return false; // cannot hold nothing
	if (contiguous())
		return ref->map_all(translate_coord(1,1), translate_coord(nrows,ncols));
	// non-contiguous, find the min and max offset of the region to load
	off_t min_off = translate_coord(1,1);
	off_t max_off = min_off;
	for (int j=1; j<=ncols; j++) for (int i=1; i<=nrows; i++) {
		off_t offset = translate_coord(i,j);
		if (min_off>offset) min_off=offset;
		if (max_off<offset) max_off=offset;
	}
	return ref->map_all(min_off, max_off);
}

bool Matrix::release() const {
	if (!(*this)) return true;
	return ref && ref->map_buffer();
}


// More complex indexing
Matrix Matrix::operator()(const Vector& indices, int c1, int c2) const
{
	if (!indices) return Matrix(); // want nothing? give nothing...
#ifndef NDEBUG // so, bound checking in action
	assert(c1>=1 && c1<=c2 && c2<=ncols); // handle null matrix case
	int index;
	for (int i=1; i<=indices.rows(); i++) {
		index = (int)indices(i);
		assert(index>=1 && index<=nrows);
	}
#endif
	// create a clone first, and add column translate to transfo list
	// if necessary, and set the correct dimension for the rows
	Matrix res(1, indices.rows(), c1, c2, this);
	// Set the indexing for the retained rows
	res.coord_transfo = new TransfoList(RowIndex, res.coord_transfo, indices);
	return res;
}

Matrix Matrix::operator()(int r1, int r2, const Vector& indices) const
{
	if (!indices) return Matrix(); // want nothing? give nothing...
#ifndef NDEBUG // so, bound checking in action
	assert(r1>=1 && r1<=r2 && r2<=nrows); // handle null matrix case
	int index;
	for (int i=1; i<=indices.rows(); i++) {
		index = (int)indices(i);
		assert(index>=1 && index<=ncols);
	}
#endif
	// create a clone first, and add row translate to transfo list
	// if necessary, and set the correct dimension for the columns
	Matrix res(r1, r2, 1, indices.rows(), this);
	// Set the indexing for the retained columns
	res.coord_transfo = new TransfoList(ColIndex, res.coord_transfo, indices);
	return res;
}

Matrix Matrix::operator()(IndexRange i, int c1, int c2) const
{
	assert(c1>=1 && c1<=c2 && c2<=ncols); // handle null matrix case
	if (i == all) // simple submatrix extraction
		return (*this)(1, nrows, c1, c2);
	assert(i == boundary); // only known type remaining
	Vector v(2); v(1) = 1; v(2) = nrows;
	// create a clone first, and add column translate to transfo list
	// if necessary, and set the correct dimension for the rows
	Matrix res(1, 2, c1, c2, this);
	// Set the indexing for the retained rows
	res.coord_transfo = new TransfoList(RowIndex, res.coord_transfo, v);
	return res;
}

Matrix Matrix::operator()(int r1, int r2, IndexRange i) const
{
	assert(r1>=1 && r1<=r2 && r2<=nrows); // handle null matrix case
	if (i == all) // simple submatrix extraction
		return (*this)(r1, r2, 1, ncols);
	assert(i == boundary); // only known type remaining
	Vector v(2); v(1) = 1; v(2) = ncols;
	// create a clone first, and add row translate to transfo list
	// if necessary, and set the correct dimension for the columns
	Matrix res(r1, r2, 1, 2, this);
	// Set the indexing for the retained columns
	res.coord_transfo = new TransfoList(ColIndex, res.coord_transfo, v);
	return res;
}

Matrix Matrix::operator()(const Vector& indices, IndexRange i) const
{
	if (i==all) return (*this)(indices, 1, ncols);
	assert(i == boundary); // only known type remaining
	Matrix res = (*this)(indices, 1, ncols); // take all columns
	Vector v(2); v(1) = 1; v(2) = ncols;     // retain only the boundary
	return res(1, 2, v);
}

Matrix Matrix::operator()(IndexRange i, const Vector& indices) const
{
	if (i==all) return (*this)(1, nrows, indices);
	assert(i == boundary); // only known type remaining
	Matrix res = (*this)(1, nrows, indices); // take all rows
	Vector v(2); v(1) = 1; v(2) = nrows;     // retain only the boundary
	return res(v, 1, 2);
}

// Other indexing modes
Vector Vector::operator()(const Vector& indices) const
{
	if (!indices) return Vector(); // want nothing? give nothing...
#ifndef NDEBUG // so, bound checking in action
	int index;
	for (int i=1; i<=indices.rows(); i++) {
		index = (int)indices(i);
		assert(index>=1 && index<=nrows);
	}
#endif
	// create a clone first, and set the correct dimension for the rows
	Vector res(1, indices.rows(), this);
	// Set the indexing for the retained rows
	res.coord_transfo = new TransfoList(RowIndex, res.coord_transfo, indices);
	return res;
}

Vector Vector::operator()(IndexRange i) const
{
	if (i == all) return (*this); // Hmm, the progress in computer science is impressive
	assert(i == boundary); // only known type remaining
	Vector v(2); v(1) = 1; v(2) = nrows;
	// create a clone first, and set the correct dimension for the rows
	Vector res(1, 2, this);
	// Set the indexing for the retained rows
	res.coord_transfo = new TransfoList(RowIndex, res.coord_transfo, v);
	return res;
}

// Unfinished iterator class slowing things down and not strictly necessary
// Comment it out
/*
Iterator::Iterator(const Matrix& m, int _r, int _c, int _endr, int _endc)
	mat(m), r(_r), c(_c), endr(_endr), endc(_endc)
{
}

Iterator& Iterator::operator++() // prefix
{
	if (!r || !c) return; // at the end, stay at the end
	if (++r > endr) {
		r = 1;
		if (++c > endc) {r=0; c=0;} // stay at the end
	}
}

Iterator& Iterator::operator--() // prefix
{
	if (r==1 && c==1) return; // at the begining, stay at the begining
	if (--r < 1) {
		r = endr;
		if (--c < 1) {r=1; c=1;} // stay at the begining
	}
}

MatrixType& Iterator::operator*() const // to access the current value
{
	if (!r || !c) return mat(endr, endc); // at the end
	return mat(r,c);
}

// Iterator functions
Iterator Matrix::begin() const
{

}

Iterator Matrix::end() const
{
}
*/

ostream& operator<<(ostream& o, const Matrix& mat)
{
	int m=mat.rows(), n=mat.cols();
	for (int i=1; i<=m; i++) {
		for (int j=1; j<n; j++) o << mat(i,j) << " ";
		o << mat(i,n) << endl;
	}
	return o;
}

istream& operator>>(istream& s, const Matrix& mat)
{
	int m=mat.rows(), n=mat.cols();
	for (int i=1; i<=m; i++) for (int j=1; j<=n; j++)
		if (s && !s.eof()) s >> mat(i,j);
	return s;
}

Matrix operator*(const Matrix& mat1, const Matrix& mat2)
{
	if (!mat1 || !mat2) return Matrix();
	int m = mat1.rows();
	int n = mat1.cols();
	assert(n==mat2.rows());
	int p = mat2.cols();
	Matrix res(m, p, _temp_size, _temp_size?tmpnam(0):"", true);
	register MatrixType value;
	for (int i=1; i<=m; i++) {
		for (int j=1; j<=p; j++) {
			value = 0;
			for (int k=1; k<=n; k++) value += mat1(i,k) * mat2(k,j);
			res(i,j) = value;
		}
	}
	return res;
}

static MatrixType fast_exp(const MatrixType& x, int n) {
	if (n==2) return x*x; // not necessary, but so common the speed gain is appreciable
	if (n==1) return x; // not so common, but recursion might end here
	if (n<=0) return 1.0; // do this test after because even less common now that x==2 can abort recursion!
	if (n&1) return fast_exp(x*x, n>>1) * x; // n is odd
	return fast_exp(x*x, n>>1); // n is even
}

// element fast exponentiation
Matrix pow(const Matrix& a, int exponent)
{
	if (!a) return Matrix();
	int m = a.rows();
	int n = a.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) {
		for (int i=1; i<=m; i++) {
			res(i,j) = fast_exp(a(i,j),exponent);
		}
	}
	return res;
}

// multiply is the element-to-element multiplication.
Matrix multiply(const Matrix& a, const Matrix& b)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	if (!a || !b) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) {
		for (int i=1; i<=m; i++) {
			res(i,j) = a(i,j) * b(i,j);
		}
	}
	return res;
}

// divide is the element-to-element division.
Matrix divide(const Matrix& a, const Matrix& b)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	if (!a || !b) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) {
		for (int i=1; i<=m; i++) {
			res(i,j) = a(i,j) / b(i,j);
		}
	}
	return res;
}

// replace *= for the element-to-element multiplication.
Matrix Matrix::multiply(const Matrix& b)
{
	assert(nrows==b.rows() && ncols==b.cols());
	if (!b) return Matrix();
	for (int j=1; j<=ncols; j++) {
		for (int i=1; i<=nrows; i++) {
			(*this)(i,j) *= b(i,j);
		}
	}
	return (*this);
}

// replace /= for the element-to-element division.
Matrix Matrix::divide(const Matrix& b)
{
	assert(nrows==b.rows() && ncols==b.cols());
	assert((bool)b); // 0 / 0 undefined
	for (int j=1; j<=ncols; j++) {
		for (int i=1; i<=nrows; i++) {
			(*this)(i,j) /= b(i,j);
		}
	}
	return (*this);
}

// dot-product
MatrixType dot(const Matrix& a, const Matrix& b)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	if (!a || !b) return 0;
	MatrixType res = 0;
	for (int j=1; j<=n; j++) {
		for (int i=1; i<=m; i++) {
			res += a(i,j) * b(i,j);
		}
	}
	return res;
}

// Special ops
Matrix Matrix::operator*=(const Matrix& a)
{
	assert(nrows==ncols && nrows==a.nrows && ncols==a.ncols);
	if (!(*this)) return Matrix();
	if (a.ref != ref) {
		// we need to store only one vector at a time
		Vector tmp(ncols, _temp_size, _temp_size?tmpnam(0):"", true);
		register MatrixType value;
		for (int i=1; i<=nrows; i++) {
			for (int j=1; j<=ncols; j++) {
				value = 0;
				for (int k=1; k<=ncols; k++) value += (*this)(i,k) * a(k,j);
				tmp(j) = value;
			}
			// put it back in our array
			for (int k=1; k<=ncols; k++) (*this)(i,k) = tmp(k);
		}
		return *this;
	}
	// possible overlap, use full matrix
	Matrix tmp(nrows, ncols, _temp_size, _temp_size?tmpnam(0):"", true);
	register MatrixType value;
	for (int i=1; i<=nrows; i++) {
		for (int j=1; j<=ncols; j++) {
			value = 0;
			for (int k=1; k<=ncols; k++) value += (*this)(i,k) * a(k,j);
			tmp(i,j) = value;
		}
	}
	for (int i=1; i<=nrows; i++) for (int j=1; j<=ncols; j++)
		(*this)(i,j) = tmp(i,j);
	return *this;
}
bool Matrix::operator==(const Matrix& a) const
{
	if (nrows!=a.nrows || ncols!=a.ncols) return false;
	for (int i=1; i<=nrows; i++) for (int j=1; j<=ncols; j++)
		if ((*this)(i,j)!=a(i,j)) return false;
	return true;
}
bool Matrix::operator!=(const Matrix& a) const
{
	return !(*this == a);
}
Matrix& Matrix::operator++() // prefix
{
	for (int i=1; i<=nrows; i++) for (int j=1; j<=ncols; j++) ++(*this)(i,j);
	return *this;
}
Matrix Matrix::operator++(int) // postfix
{
	for (int i=1; i<=nrows; i++) for (int j=1; j<=ncols; j++) (*this)(i,j)++;
	return *this;
}
Matrix& Matrix::operator--() // prefix
{
	for (int i=1; i<=nrows; i++) for (int j=1; j<=ncols; j++) --(*this)(i,j);
	return *this;
}
Matrix Matrix::operator--(int) // postfix
{
	for (int i=1; i<=nrows; i++) for (int j=1; j<=ncols; j++) (*this)(i,j)--;
	return *this;
}
// unary -
Matrix Matrix::operator-()
{
	if (!(*this)) return Matrix();
	Matrix res(nrows, ncols, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int i=1; i<=nrows; i++) for (int j=1; j<=ncols; j++)
		res(i,j) = -(*this)(i,j);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType), const Matrix& a) {
	if (!a) return Matrix();
	int m = a.rows();
	int n = a.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j));
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType), const Matrix& a, const Matrix& b)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	if (!a || !b) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b(i,j));
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType), const Matrix& a, const MatrixType& b)
{
	if (!a) return Matrix();
	int m = a.rows();
	int n = a.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType), const MatrixType& a, const Matrix& b)
{
	if (!b) return Matrix();
	int m = b.rows();
	int n = b.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a,b(i,j));
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType), const Matrix& a, const Matrix& b, const Matrix& c)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	assert(m==c.rows() && n==c.cols());
	if (!a || !b || !c) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b(i,j),c(i,j));
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType), const Matrix& a, const Matrix& b, const MatrixType& c)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	if (!a || !b) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b(i,j),c);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType), const Matrix& a, const MatrixType& b, const Matrix& c)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==c.rows() && n==c.cols());
	if (!a || !c) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b,c(i,j));
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType), const Matrix& a, const MatrixType& b, const MatrixType& c)
{
	if (!a) return Matrix();
	int m = a.rows();
	int n = a.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b,c);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType), const MatrixType& a, const Matrix& b, const Matrix& c)
{
	int m = b.rows();
	int n = b.cols();
	assert(m==c.rows() && n==c.cols());
	if (!b || !c) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a,b(i,j),c(i,j));
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType), const MatrixType& a, const Matrix& b, const MatrixType& c)
{
	if (!b) return Matrix();
	int m = b.rows();
	int n = b.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a,b(i,j),c);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType), const MatrixType& a, const MatrixType& b, const Matrix& c)
{
	if (!c) return Matrix();
	int m = c.rows();
	int n = c.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a,b,c(i,j));
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,void*), const Matrix& a, void* arg) {
	if (!a) return Matrix();
	int m = a.rows();
	int n = a.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,void*), const Matrix& a, const Matrix& b, void* arg)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	if (!a || !b) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b(i,j),arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,void*), const Matrix& a, const MatrixType& b, void* arg)
{
	if (!a) return Matrix();
	int m = a.rows();
	int n = a.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b,arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,void*), const MatrixType& a, const Matrix& b, void* arg)
{
	if (!b) return Matrix();
	int m = b.rows();
	int n = b.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a,b(i,j),arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType,void*), const Matrix& a, const Matrix& b, const Matrix& c, void* arg)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	assert(m==c.rows() && n==c.cols());
	if (!a || !b || !c) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b(i,j),c(i,j),arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType,void*), const Matrix& a, const Matrix& b, const MatrixType& c, void* arg)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	if (!a || !b) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b(i,j),c,arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType,void*), const Matrix& a, const MatrixType& b, const Matrix& c, void* arg)
{
	int m = a.rows();
	int n = a.cols();
	assert(m==c.rows() && n==c.cols());
	if (!a || !c) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b,c(i,j),arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType,void*), const Matrix& a, const MatrixType& b, const MatrixType& c, void* arg)
{
	if (!a) return Matrix();
	int m = a.rows();
	int n = a.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a(i,j),b,c,arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType,void*), const MatrixType& a, const Matrix& b, const Matrix& c, void* arg)
{
	int m = b.rows();
	int n = b.cols();
	assert(m==c.rows() && n==c.cols());
	if (!b || !c) return Matrix();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a,b(i,j),c(i,j),arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType,void*), const MatrixType& a, const Matrix& b, const MatrixType& c, void* arg)
{
	if (!b) return Matrix();
	int m = b.rows();
	int n = b.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a,b(i,j),c,arg);
	return res;
}

Matrix map(MatrixType (*func)(MatrixType,MatrixType,MatrixType,void*), const MatrixType& a, const MatrixType& b, const Matrix& c, void* arg)
{
	if (!c) return Matrix();
	int m = c.rows();
	int n = c.cols();
	Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
		res(i,j) = func(a,b,c(i,j),arg);
	return res;
}

// Create a matrix whose elements are generated by a user function
Matrix generate(MatrixType (*func)(int,int), int rows, int cols, size_t bsize=0, const string& filename="", bool auto_del=false)
{
	assert(rows>0 && cols>0);
	Matrix ret(rows, cols, bsize, filename, auto_del);
	if (!ret) return Matrix();
	for (int j=1; j<=cols; j++) for (int i=1; i<=rows; i++) ret(i,j) = func(i,j);
	return ret;
}

#include "CheapMatrixOps.source"

} // end of namespace


/*
using namespace CheapMatrix;

static MatrixType my_generator(int i, int j) {return i-j;}

main()
{
//	Matrix a(100,100);
//	for (int j=1; j<=a.cols(); j++) for (int i=1; i<=a.rows(); i++) a(i,j)=1.0;
//	cout<<a;

//	Matrix::temporary_buffer(100);
	Matrix M(3,2);
	M = 1, 2,
	    3, 4,
	    5, 6;
	Matrix P(3,2);
	P = 2, 1,
	    1, 3,
	    1, 2;
	P *= 2.0;
	Matrix res = fabs(P-M);
	cout << ++res.transpose() << endl; // 4 2 4 endl 1 3 3 endl endl
//	if (!P.reshape(2,3)) cerr<<"error"<<endl;
	P = P.reshape(2,3);
	cout << M * P << endl; // 8 6 14 endl 20 14 34 endl 32 22 54 endl endl
	Vector V(2); // column vector
	V = 1, 1;
	cout << V << endl;
	cout << M*V << endl;
	Matrix S(2,2), Q(2,2);
	S = 1, 2,
	    3, 4;
	Q = 5, 6,
	    7, 8;
	S *= Q;
	cout << S; // 19 22 endl 43 50 endl
	S = 1, 2,
	    3, 4;
	S *= S;
	cout << endl << S;
	Matrix A(4,4);
	A = 1, 1, 1, 1,
	    1, 1, 1, 1,
	    1, 1, 1, 1,
	    1, 1, 1, 1;
	Matrix B = A(2,3,2,3);
	B -= 2.0;
	Matrix C = A(2,4,2,4);
	C = C.transpose();
	Matrix D(3,3); D=1,2,3,4,5,6,7,8,9;
	C += D;
	cout << A;
//	if (A.reshape(12,12)) cerr<<"gasp!"<<endl; else cout<<"bool test OK"<<endl;
	Matrix BB = A;
//	cout << BB.flip_rows(1,2).flip_col_range(1,3) <<endl;
//	BB(2,3,2,3).copy(Q);
	BB.flip_rows(1,2).flip_col_range(1,3)(2,3,2,3).copy(Q);
	cout << A;
	A(1,2,1,2).copy(A(1,2,2,3));
	A(2,4,2,3).copy(A(2,4,1,2));
	cout << endl << A;

	cout << endl;
	cout << A[0][0] << " " << A[0][1] << " " << A[0][2] << " " << A[0][3] << endl;
	cout << A[1][0] << " " << A[1][1] << " " << A[1][2] << " " << A[1][3] << endl;
	cout << A[2][0] << " " << A[2][1] << " " << A[2][2] << " " << A[2][3] << endl;
	cout << A[3][0] << " " << A[3][1] << " " << A[3][2] << " " << A[3][3] << endl;

	Vector VV(3);
	VV = 0,1,2;
	Vector W = VV + VV;
	cout<<endl<<W<<endl;

	cout << map(pow,3.0,VV) << sqrt(pow(3.0,VV))+1 << endl;

	Matrix MapTest = generate(my_generator,3,3);
	cout << MapTest << endl;
	cout << MapTest.map(fabs) << endl;
	cout << MapTest.fill(4.2) << endl;
	cout << MapTest.fill(my_generator) << endl;
}

*/